#ifndef Analysis_h 
#define Analysis_h
/*****************************************************************************
 * Copyright (C) 2009-2014    this file is part of the NPTool Project        *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Adrien MATTA  contact address: a.matta@surrey.ac.uk      *
 *                                                                           *
 * Creation Date  : march 2025                                               *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 * Class describing the property of an Analysis object                       *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
#include"NPVAnalysis.h"
#include"NPEnergyLoss.h"
#include"NPReaction.h"
#include"RootOutput.h"
#include"RootInput.h"
#include "TSharcPhysics.h"
#include "TTigressPhysics.h"
#include "TInitialConditions.h"
#include <TRandom3.h>
#include <TVector3.h>
#include <TMath.h>
#include<TCutG.h>

#include<fstream>
#include<vector>
class Analysis: public NPL::VAnalysis{
  public:
    Analysis();
    ~Analysis();

  public: 
    void Init();
    void TreatEvent();
    void End();

  void InitOutputBranch();
  void InitInputBranch();
  void ReInitValue();
  static NPL::VAnalysis* Construct();
 
  private:
  double Ex;
  double ELab;
  double ThetaLab;
  double ThetaCM;
  NPL::Reaction* myReaction;
TInitialConditions* myInit ;
  //	Energy loss table: the G4Table are generated by the simulation
  EnergyLoss LightCD2;
  EnergyLoss LightAl;
  EnergyLoss LightSi;
  EnergyLoss BeamCD2;
  TVector3 BeamDirection;
  TVector3 TargetPosition;


  double TargetThickness ;
  // Beam Energy
  double OriginalBeamEnergy ; // AMEV
                                                           // intermediate variable
  TRandom3 Rand ;
  int DetectorNumber  ;
  int RunNumber;
  int RunNumberMinor;
  double ThetaNormalTarget;
  double Energy ;
  
  double ThetaSharcSurface ;
  double X_Sharc ;
  double Y_Sharc ;
  double Z_Sharc ;
  double X_Trifoil;
  double Y_Trifoil;
  double ThetaDetector;
  double Si_E_Sharc ;
  double E_Sharc ;
  double Distance;
  TSharcPhysics* Sharc;
  TTigressPhysics* Tigress;

  std::vector<TCutG*> cut_ex5; 
  std::ofstream box_pos;
  std::ofstream qqq_pos;
  TCutG* cut_dd;
  TCutG* cut_deuton_d10;
  TCutG* cut_proton_d10;
  TCutG* cut_deuton_d12;
  TCutG* cut_proton_d12;
  std::ofstream PADFile;

};
#endif
